#!/usr/bin/env bash

script_name="$(basename $(realpath $0))"
working_dir="$(dirname $(realpath $0))"

conf_file="${working_dir}/conf/${script_name%%.*}.conf"

LOCK_DIR="/tmp/$(basename $0).$$.LOCK"
log_file="${LOCK_DIR}/${script_name%%.*}.log"

backup_date="$(date +%Y-%m-%d-%H%M)"
declare -a files_backuped_arr=()
declare -a dirs_backuped_arr=()

# ----------
# Base Function(s)
# ----------

usage() {


   [[ -n  "$1" ]] && error "$1"


   [[ $terminal ]] && echo -e "
\033[1mUsage:\033[m

   $(basename $0) [OPTION [OPTION ..

\033[1mDescription\033[m

   <Some Description>

\033[1mOptions\033[m

   <List Options>

\033[1mFiles\033[m

   $conf_file: Configuration file

\033[1mExample:\033[m
   
   <description example>

      $(basename $0) .. <comand example>

   <description another example>
   
      $(basename $0) .. <command another example>

"

   clean_up 1

}

clean_up() {

   # Perform program exit housekeeping
   rm -rf "$LOCK_DIR"
   blank_line
   exit $1
}


echononl(){
   if $terminal ; then
      echo X\\c > /tmp/shprompt$$
      if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
         echo -e -n "   $*\\c" 1>&2
      else
         echo -e -n "   $*" 1>&2
      fi
      rm /tmp/shprompt$$
   fi
}


fatal(){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mFatal\033[m ] $*"
   else
      echo -e "   [ Fatal ] $*"
   fi
   echo ""
   if $terminal ; then
      echo -e "             \033[1mScript terminated\033[m.."
   else
      echo -e "             Script terminated.."
   fi
   echo ""
   rm -rf $LOCK_DIR
   exit 1
}

error (){
   echo ""
   if $terminal ; then
      echo -e "   [ \033[31m\033[1mError\033[m ] $*"
   else
      echo "   [ Error ] $*"
   fi
   echo ""
}

warn (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[33m\033[1mWarn\033[m ]  $*"
      else
         echo "   [ Warn ]  $*"
      fi
      echo ""
   fi
}

info (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mInfo\033[m ]  $*"
      else
         echo "   [ Info ]  $*"
      fi
      echo ""
   fi
}

ok (){
   if $LOGGING || $terminal ; then
      echo ""
      if $terminal ; then
         echo -e "   [ \033[32m\033[1mOk\033[m ]    $*"
      else
         echo "   [ Ok ]    $*"
      fi
      echo ""
   fi
}

echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mskipped\033[m ]"
   fi
}
echo_wait(){
   if $terminal ; then
      echo -en "\033[75G[ \033[5m\033[1m...\033[m ]"
   fi
}

trim() {
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}

# - Backup file or directory
# -
backup() {

   is_directory=false

   if [[ -z "$1" ]] ; then
      error "No file/directory for backup given!"
      return
   elif [[ -h "$1" ]] ; then
      _source="$(realpath $1)"
      warn "'$1' is a symlink to $_source."
   elif [[ ! -d "$1" ]] && [[ ! -f "$1" ]] ; then
      warn "'$1' not found. Backup skipped."
      return
   else
      _source="$(realpath $1)"
   fi

   if [[ -d "$_source" ]]; then
      is_directory=true
      echononl "Backup directory '$_source' .."
   else
      echononl "Backup file '$_source' .."
   fi

   cp -a "$_source" "${_source}.$backup_date" > $log_file 2>&1
   if [[ $? -eq 0 ]] ; then
      echo_ok
      if $is_directory ; then
         dirs_backuped_arr+=("$_source")
      else
         files_backuped_arr+=("$_source")
      fi
   else
      echo_failed
      error "$(cat "$log_file")"
   fi
}


# - Remove script generated backups, if source wasn't changed
# -
rm_unchanged_backup() {

   if [[ ${#files_backuped_arr[@]} -gt 0 ]] ; then
      for _file in "${files_backuped_arr[@]}" ; do
         if  $(diff "$_file" "${_file}.$backup_date" > /dev/null 2>&1) ; then
            echononl "File '$(basename "${_file}")' wasn't changed.\n   Delete the previous generated backup. .."
            rm "${_file}.$backup_date" > "$log_file" 2>&1
            if [[ $? -eq 0 ]]; then
               echo_ok
            else
               echo_failed
               error "$(cat "$log_file")"
            fi
            blank_line
         fi
      done
   fi
   if [[ ${#dirs_backuped_arr[@]} -gt 0 ]] ; then
      for _dir in "${dirs_backuped_arr[@]}" ; do
         if $(diff -Nur "$_dir" "${_dir}.$backup_date" > /dev/null 2>&1) ; then
            echononl "Directory '$(basename "${_dir}")' wasn't changed.\n   Delete the previous generated backup. .."
            rm -rf "${_dir}.$backup_date" > "$log_file" 2>&1
            if [[ $? -eq 0 ]]; then
               echo_ok
            else
               echo_failed
               error "$(cat "$log_file")"
            fi
            blank_line
         fi
      done
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

# - Create lock directory '$LOCK_DIR"
#
mkdir "$LOCK_DIR"


# ----------
# - Some checks ..
# ----------

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
   terminal=true
else
   terminal=false
fi

# -Is systemd supported on this system?
# -
systemd_supported=false
systemd=$(which systemd)
systemctl=$(which systemctl)

if [[ -n "$systemd" ]] && [[ -n "$systemctl" ]] ; then
   systemd_supported=true
fi

# - Print help?
# -
if [[ "$(trim $*)" = "-h" ]] || [[ "$(trim $*)" = "--help" ]] ; then
    usage
fi

if [[ -z "$(which basename)" ]]; then
   fatal 'It seems "basename" is not installed, but needed!'
fi

if [[ -z "$(which realpath)" ]]; then
   fatal 'It seems "realpath" is not installed, but needed!'
fi


# ==========
# - Begin Main Script
# ==========

# ----------
# - Headline
# ----------

if $terminal ; then
   echo ""
   echo -e "\033[1m----------\033[m"
   echo -e "\033[32m\033[1mRunning script \033[m\033[1m$script_name\033[32m .. \033[m"
   echo -e "\033[1m----------\033[m"
fi


# ----------
#  Read Configurations from $conf_file
# ----------


# - Give your default values here
# -
LOGGING=false
BATCH_MODE=false
PARAMETER_1="WERT_1"
PARAMETER_2="WERT_2"

if [[ -f "$conf_file" ]]; then
   source "$conf_file"
else
   warn "No configuration file '$conf_file' present.\n
             Loading default values.."
fi


# ----------
# - Read commandline parameter
# ----------

while getopts bh opt ; do
   case $opt in
      b)    BATCH_MODE=true
            ;;
      h)    usage
            ;;
      \?)   usage ;;
   esac
done


# ----------
# - Read in parameters given by user
# ----------

if $terminal && ! $BATCH_MODE ; then

   echo ""
   echo ""
   echo -e "   \033[1mRead in parameters given by user ..\033[m"
   echo ""

   echo ""
   echo -e "\033[32m--\033[m"
   echo ""
   echo "   PARAMETER 1 ...."
   echo ""
   echo -e "   Type \033[33m<return>\033[m to accept the default."
   echo ""
   echo ""
   _PARAMETER_1=$PARAMETER_1
   PARAMETER_1=""
   echononl "PARAMETER 1 [\033[1m$_PARAMETER_1\033[m]: "
   read PARAMETER_1
   if [[ -z "$PARAMETER_1" ]] ; then
      PARAMETER_1=$_PARAMETER_1
   fi

   echo ""
   echo -e "\033[32m--\033[m"
   echo ""
   echo "   Blank separated list of PARAMETER 2"
   echo ""
   if [[ -n "$PARAMETER_2" ]]; then

      echo -e "   Type \033[33m<return>\033[m to accept the default."
      echo ""
      echo ""

      _PARAMETER_2=$PARAMETER_2
      PARAMETER_2=""

      echononl "PARAMETER 2 [\033[1m$_PARAMETER_2\033[m]: "
      read PARAMETER_2
      if [[ -z "$PARAMETER_2" ]] ; then
         PARAMETER_2=$_PARAMETER_2
      fi
   else
      echo ""

      echononl "\033[1mPARAMETER 2:\033[m "
      read PARAMETER_2
      while [[ -z "$(trim "$PARAMETER_2")" ]]; do
         echo "Entry for  PARAMETER 2 is required! Try again.."
         echononl "\033[1mPARAMETER 2: \033[m"
         read PARAMETER_2
      done
   fi

   echo ""
   echo ""
   echo -e "   \033[32m\033[1mParameter Summary:\033[m"
	echo ""

   echo "   PARAMETER 1................: $PARAMETER_1"
   echo "   PARAMETER 2................: $PARAMETER_2"

   echo ""
   echononl "Continue with this parameters? [\033[1myes/no\033[m]: "
   read OK
   while [[ "${OK,,}" != "yes" ]] && [[ "${OK,,}" != "no" ]] ; do
      echononl "\033[33mWrong entry!\033[m [\033[1myes/no\033[m]: "
      read OK
   done
   [[ "${OK,,}" = "yes" ]] || fatal "Canceled by user input."

fi


# ----------
# - Some pre-script tasks ..
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mDoing some pre-script tasks ..\033[m"
   echo ""
fi

echononl "-- All is fine --"
echo_ok


# ----------
# - Main part of script
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mMain part of script ..\033[m"
   echo ""
fi


# ----------
# - Some post-script tasks ..
# ----------

if $terminal ; then
   echo ""
   echo ""
   echo -e "   \033[1mDoing some post-script tasks ..\033[m"
   echo ""
fi


# - Remove generated backup file if source wasn't changed.
# -
rm_unchanged_backup

clean_up 0
